
import ttk
import os
import os.path
import sys

from pymol import cmd
import pymol

from tkinter import *
import tkFileDialog
import tkMessageBox

import Surface
import MolecularSurfaceView
import ViewerCallback
import DecalList

class GRAPE:
	viewer = None
	viewcallback = None
	#Some UI elements
	radiosel = 0
	stickersel = [None, None, None, None]
	visiblesel = []
	
	def __init__(self):		
		pass
	
		
		
		
	def buildInterface(self, viewer):		
		chaincount = len(viewer.surfaceCollections)
		print "Chaincount: ",
		print chaincount
		for i in range(0, chaincount):
			self.visiblesel.append([BooleanVar(value = True), BooleanVar(value = True)])
			ttk.Label(self.mainframe, text = "Chain " + str(i)).grid(row = i+5, column = 0, sticky = W)			
			ttk.Checkbutton(self.mainframe, text = "Visible", variable = self.visiblesel[i][0],  command = self.visChanged).grid(row = i+5, column = 1, sticky = W)
			if viewer.surfaceCollections[i].canDrawAbstracted():
				ttk.Checkbutton(self.mainframe, text = "Abstracted", variable = self.visiblesel[i][1],  command = self.visChanged).grid(row = i+5, column = 2, sticky = W)
		self.setVisible()
			
	def visChanged(self, *args):
		#Update the visible items
		if self.viewer:
			self.setVisible()
		cmd.refresh()
		
		
	def radioChanged(self, *args):
		print self.radiosel.get() #Pull the intvar.
		if self.viewer:
			selval = self.radiosel.get()
			self.setColor(selval)	
		cmd.refresh()
			
	def checkChanged(self, *args):	
		self.setDecal()
		cmd.refresh()
		
	def setVisible(self):
		chaincount = len(self.viewer.surfaceCollections)
		for i in range(0, chaincount):
			self.viewer.setVisibility(i, self.visiblesel[i][0].get())
			if self.viewer.surfaceCollections[i].canDrawAbstracted():
				self.viewer.setAbstracted(i, self.visiblesel[i][1].get())
		
	def setDecal(self):
		chaincount = 0
		if self.viewer:
			chaincount = len(self.viewer.surfaceCollections)
		else:
			return
			
		for i in range(0, chaincount):
			#H-Bonds			
			self.viewer.setStickerVis(i, DecalList.DecalListType_h_bond_acceptor, self.stickersel[0].get())
			self.viewer.setStickerVis(i, DecalList.DecalListType_h_bond_donor, self.stickersel[0].get())
			#Ligands
			self.viewer.setStickerVis(i, DecalList.DecalListType_pocket, self.stickersel[1].get())
			#Shadows
			self.viewer.setStickerVis(i, DecalList.DecalListType_shadow, self.stickersel[2].get())
			#Peak/Bowls
			self.viewer.setStickerVis(i, DecalList.DecalListType_sanded, self.stickersel[3].get())	
		
	def setColor(self, value):
		#Technically, these are already hardcoded to these values, but let's be classy here.
		chaincount = 0
		if self.viewer:
			chaincount = len(self.viewer.surfaceCollections)
		else:
			return
			
		for i in range(0, chaincount):
			if (value == 0):
				self.viewer.setSurfaceColor(i, Surface.showSurfaceColor_none)
			if (value == 1):
				self.viewer.setSurfaceColor(i, Surface.showSurfaceColor_bychain)
			if (value == 2):
				self.viewer.setSurfaceColor(i, Surface.showSurfaceColor_charge)				
			if (value == 3):
				self.viewer.setSurfaceColor(i, Surface.showSurfaceColor_hydropathy)
		
		
		
	def loadPrompt(self, *args):
		loadlocation = tkFileDialog.askopenfilename(defaultextension = '' , initialdir = 'C:\\', initialfile = '', parent = self.root, filetypes=[("GRAPE Zip data",".zip")], title = 'Load file')
		if (loadlocation != None):
			try:
				#Need to parse the name....
				[pathname, basename] = os.path.split(loadlocation)
				[basename, ext] = os.path.splitext(basename)
				if (len(pathname) > 0) and ((len(basename) > 0)):
					self.viewer = MolecularSurfaceView.MolecularSurfaceView(pathname)
					self.viewer.setBaseFileName(basename)
				else:
					return
				self.viewer.loadSurfaces()			
				self.setColor(self.radiosel.get())		
				self.setDecal()
				self.buildInterface(self.viewer)
				if self.viewcallback == None:
					self.viewcallback = ViewerCallback.ViewerCallback(self.viewer)		
					cmd.load_callback(self.viewcallback, 'GRAPE Visualization')
				else:
					self.viewcallback.setView(self.viewer)
				cmd.refresh()
			except Exception as e:
				pass		
		
			
			
					
	def startTk(self, tkobject):
		self.root = tkobject
		self.root.title("GRAPE plugin")

		self.mainframe = ttk.Frame(self.root, padding="20 10 20 10")
		self.mainframe.grid(column=0, row=0, sticky=(N, W, E, S))
		self.mainframe.columnconfigure(0, weight=1)
		self.mainframe.rowconfigure(0, weight=1)

		startup = ''#StringVar()
		self.menubar = Menu(self.root)
		self.mainfileMenu = Menu(self.menubar, tearoff=0)
		self.mainfileMenu.add_command(label="Load Molecule", command=self.loadPrompt)
		
		self.menubar.add_cascade(label="File", menu=self.mainfileMenu)	
		self.root.config(menu=self.menubar)
		
					
		#ttk.Label(self.mainframe, text="Welcome to GRAPE").grid(column=2, row=2, sticky=W)
		
		self.radiosel = IntVar()
		
		coloropts = ["None", "By Chain", "Electrostatics", "Hydropathy"]
		stickeropts = ["H-Bonds", "Detected Pockets", "Interfaces", "Peaks/Bowls"]
		#Default view is electrostatics.
		self.radiosel.set(2)		
		for i in range(0, 4):
			self.stickersel[i] = BooleanVar(value = False)
		
		ttk.Label(self.mainframe, text = "Color: ").grid(row = 3, column = 0, sticky = W)
		ttk.Radiobutton(self.mainframe, text = "None", variable = self.radiosel, value = 0, command = self.radioChanged).grid(row = 3, column = 1, sticky = W)
		ttk.Radiobutton(self.mainframe, text = "By Chain", variable = self.radiosel, value = 1,  command = self.radioChanged).grid(row = 3, column = 2, sticky = W)
		ttk.Radiobutton(self.mainframe, text = "Electrostatics", variable = self.radiosel, value = 2,  command = self.radioChanged).grid(row = 3, column = 3, sticky = W)
		ttk.Radiobutton(self.mainframe, text = "Hydropathy", variable = self.radiosel, value = 3,  command = self.radioChanged).grid(row = 3, column = 4, sticky = W)
		
		ttk.Label(self.mainframe, text = "Stickers: ").grid(row = 4, column = 0, sticky = W)
		ttk.Checkbutton(self.mainframe, text = "H-Bonds", variable = self.stickersel[0],  command = self.checkChanged).grid(row = 4, column = 1, sticky = W)
		ttk.Checkbutton(self.mainframe, text = "Detected Pockets", variable = self.stickersel[1],  command = self.checkChanged).grid(row = 4, column = 2, sticky = W)
		ttk.Checkbutton(self.mainframe, text = "Interfaces", variable = self.stickersel[2],  command = self.checkChanged).grid(row = 4, column = 3, sticky = W)
		ttk.Checkbutton(self.mainframe, text = "Peaks/Bowls", variable = self.stickersel[3],  command = self.checkChanged).grid(row = 4, column = 4, sticky = W)
		
		
		#for child in self.mainframe.winfo_children(): child.grid_configure(padx=5, pady=5)
