
import Surface
import DecalList

import sys, traceback

from pymol import cmd

#Fun parsing times
import zipfile 
import xml.etree.ElementTree
#from xml.etree.ElementTree import * 

from pymol.opengl.gl import *

class SurfaceCollection:
	visible = False	
	drawAbstract = False
	canDrawAbstract = False
	seSurface = None
	abstractSurface = None
	
	def __init__(self, canDrawAbstract, surf = None):
		self.canDrawAbstract = canDrawAbstract
		self.drawAbstract = canDrawAbstract
		self.visible = True
		if (surf != None):
			setSurface(surf)
			
			
	def getDrawAbstract(self):
		return self.drawAbstract
		
	def canDrawAbstracted(self):
		return self.canDrawAbstract
	
	def isVisible(self):
		return self.visible
		
	def setDrawAbstract(self, b):
		if (self.canDrawAbstract):
			self.drawAbstract = b
	
	def setVisibility(self, b):
		self.visible = b
		
	def isIncomplete(self):
		if (self.drawAbstract and (self.abstractSurface == None)) or \
		   ((not self.drawAbstract) and (self.seSurface == None)):
			return True
		return False

	def setSurface(self, s):
		if (self.drawAbstract):
			self.abstractSurface = s
		else:
			self.seSurface = s
			
	def getSurface(self):
		if (self.drawAbstract):
			return self.abstractSurface
		else:
			return self.seSurface
			
	def setSurfColors(self, enm):
		if (self.seSurface != None):
			self.seSurface.setColor(enm)
		if (self.abstractSurface != None):
			self.abstractSurface.setColor(enm)

class MolecularSurfaceView:
	
	SURFACE_DATA = "surfacedata"
	CHAIN_SURFACE = "chainsurface"
	curColor = Surface.showSurfaceColor_charge
	repositoryDir = None
	baseFilename = None
	surfaceCollections = []
	#cameraNeedsSet = False
	zipfiledata  = None
	
	
	#drawList = []
	
	def __init__(self, repositoryDir):
		self.repositoryDir = repositoryDir
		#ambient or headlight color? - part of joglpanel and not needed here, i guess
		self.surfaceCollections = []
		self.drawList = []
		
				
	
	def setBaseFileName(self, filename):
		self.baseFilename = filename
		#Just like a try block. Thanks Python!
		try:
			self.zipfiledata = zipfile.ZipFile(self.repositoryDir + self.baseFilename + ".zip", mode = 'r')
			print "Building surface collections..."
			self.buildSurfaceCollections()
			#cameraNeedsSet = True
		except:
			print "Bailed on loading zip file"
			
				
		
	def cleanUp(self):
		if (surfaceCollections != None):
			for surf in surfaceCollections:
				if (surf.abstractSurface != None):
					surf.abstractSurface.disposeOfResources()
				if (surf.seSurface != None):
					surf.seSurface.disposeOfResources()
		#Garbage collect this
		surfaceCollections = None
		
	
	def getSurfaceNode(self):
		#XML Parser reads "baseFilename + .xml"
		#Get the XML file out of the zip file
		
		xmlfilename = self.baseFilename + ".xml"		
		xmlstring = self.zipfiledata.read(xmlfilename)
		#xmldata is going to be a string at this point.
		#Going to make some base assumptions that are slightly different the the original
		#XML parser. Namely XPATH and the fun Node-based stuff isn't exactly the same in python
		#Yay for 30 different parser implementations and encodings.
		#ElementTree uses lots of elements. Searching an element tree only searches subnodes,
		#not itself, so we must check that too
		
		rootElement = xml.etree.ElementTree.fromstring(xmlstring)
		
		if (rootElement.tag == self.SURFACE_DATA):
			return rootElement
		children = rootElement.findall(self.SURFACE_DATA)
		if (len(children) > 0):
			return children[0]
			
		
	
	def buildSurfaceCollections(self):
		#Using xml.etree.ElementTree for simple xml parsing and whatnot
		try:
			allSurfaceData = self.getSurfaceNode()
			nl = allSurfaceData.getchildren()
			for i in range(0, len(nl)):
				if (nl[i].tag == self.CHAIN_SURFACE) and (len(nl[i].getchildren()) > 0):
					#attributes are dictionaries
					objFile = nl[i].attrib['abstract_surfacefile']
					self.surfaceCollections.append(SurfaceCollection(len(objFile) > 0))
		except Exception as e:
			return e
			print "Error in building surface collections:",
			print type(e)
			print e.args
			print e
			traceback.print_exc(file=sys.stdout)
			
	def loadSurfaces(self):
		numIncomplete = 0
		numLoaded = 0
		
		for i in range(0, len(self.surfaceCollections)):
			if self.surfaceCollections[i].isIncomplete():
				numIncomplete += 1
		
		if (numIncomplete > 0):
			for z in range(0, 1):
				allSurfaceData = self.getSurfaceNode()
				nl = allSurfaceData.getchildren()
				numChains = 0
				for i in range(0, len(nl)):
					if (nl[i].tag == self.CHAIN_SURFACE) and (len(nl[i].getchildren()) > 0):
						numChains += 1
				numIncomplete += (numChains - len(self.surfaceCollections))
				
				chainNum = 0
				for i in range(0, len(nl)):
					chainSurfaceNode = nl[i]
					if (chainSurfaceNode.tag == self.CHAIN_SURFACE) \
						and (len(chainSurfaceNode.getchildren()) > 0):
						
						if (len(self.surfaceCollections) <= chainNum):
							self.surfaceCollections.append(SurfaceCollection(true))
							
						sc = self.surfaceCollections[chainNum]
						if (sc.isIncomplete()):
							s = Surface.Surface(self.zipfiledata, \
												chainSurfaceNode, \
												chainNum, \
												sc.getDrawAbstract())
							s.buildScene()
							s.setColor(self.curColor)
							
							sc.setSurface(s)
												
					chainNum += 1
			#except Exception as e:
			#	print "Error in load surfaces:",
			#	print type(e)
			#	print e.args
			#	print e		
			#	traceback.print_exc(file=sys.stdout)
		
	def redoSceneGraph(self):
		#TODO - figure out exactly what this is.
		#This is going to update what is and is not rendered in the scene graph
		pass
		
	def prepTextures(self):
		#Do we need to 'prep' textures in the same was as jogl?			
		pass
		
	def Draw(self):		
		#This actually draws. The refresh is done by pymol. We're merely providing a rendering part

		for sc in self.surfaceCollections:		
			if (sc.isVisible()):
				surf = sc.getSurface()
				surf.Render(sc.isVisible())

		
		
	def setSurfaceColor(self, chainNum, enm):	
		self.curColor = enm
		self.surfaceCollections[chainNum].setSurfColors(enm)
		
	def setVisibility(self, chainNum, val):
		if (self.baseFilename != None):
			self.surfaceCollections[chainNum].setVisibility(val)
			#redoSceneGraph()
		
	def setAbstracted(self, chainNum, drawAbstracted):
		if (self.baseFilename != None):
			self.surfaceCollections[chainNum].setDrawAbstract(drawAbstracted)
			self.loadSurfaces()			
			#cmd.refresh()
	
	def setStickerVis(self, chainNum, type, val):
		if (type != DecalList.DecalListType_unknown):
			if self.surfaceCollections[chainNum].getDrawAbstract():
				absurf = self.surfaceCollections[chainNum].getSurface()
				absurf.setStickerTypeVisible(type, val)
				#cmd.refresh()
		
		
		