
#from OpenGL.GL import *
from pymol.opengl.gl import *
from pymol.opengl.glu import *

import zipfile
import png    #pyPNG required here.....
import array

class Texture2D:
	#Treating texture id slightly differently than jogl. Assuming only 1 GL context, so we can just store the GL ID
	texture_id = None    
	image = None
	mag_filter = GL_LINEAR
	min_filter = GL_LINEAR
	mode = GL_MODULATE
	
	rebuild_texture = True
	isDisposed = False
	
	
	zipobject = None
	texFileName = None
	needsLoaded = True
	n_rows = 0
	n_cols = 0
	wrap_s = GL_CLAMP_TO_BORDER
	wrap_t = GL_CLAMP_TO_BORDER
		
	
	
	def __init__(self, texFileName, zipobject):
		self.setMustRebuild(True)
		needsLoaded = True
		self.zipobject = zipobject
		self.texFileName = texFileName
		#Load on init instead of on call. Saves us some time when rendering
		self.loadTextureFromZip()
		

	def loadTextureFromZip(self):
		if (self.needsLoaded):
		
			#Using zipfile to extract a PNG file into PyPNG memory
			pngstring = self.zipobject.read(self.texFileName)
			reader = png.Reader(bytes = pngstring)
			imagedata = reader.read_flat()
			
			#Parsed data is formmated as: Width, Height, Pixels, Metadata
			width = imagedata[0]
			height = imagedata[1]
			pixels = imagedata[2]
			metadata = imagedata[3] #probably unused for us here....
			self.n_cols = width
			self.n_rows = height
			
					
			
			fixedpixels = []
			
			#The rows of the PNG data are upside-down from what Opengl likes. Reverse that.			
			for y in reversed(range(0, height)):
				for x in range(0, width):
					offset = x  + y * width
					offset = offset * 4
					fixedpixels.append(pixels[offset])
					fixedpixels.append(pixels[offset+1])
					fixedpixels.append(pixels[offset+2])
					fixedpixels.append(pixels[offset+3])			
			
			pixels = array.array('B', fixedpixels)
			print len(pixels)
			#print reversepixels
			self.image = pixels.tostring()
			#print self.image
			self.needsLoaded = False
		
						
		
	def setMustRebuild(self, b):
		self.rebuild_texture = b
				
	def mustRebuild(self):
		return self.rebuild_texture
				
	def activate(self):
		if (self.isDisposed):
			return False
		if (self.needsLoaded):
			self.loadTextureFromZip()
			
		glEnable(GL_TEXTURE_2D)
		tex_name = self.getTexture_name()
		glBindTexture(GL_TEXTURE_2D, tex_name)
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, self.mode )
		
		if self.mustRebuild() and self.image != None:
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
		
			#This needs to be checked out thoroughly to ensure we're using GL_BGRA correctly....
			gluBuild2DMipmaps(GL_TEXTURE_2D, 4, \
						self.n_cols, self.n_rows, \
						GL_RGBA, GL_UNSIGNED_BYTE, self.image)
			#print self.image
			#glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.n_cols, self.n_rows, 0, GL_UNSIGNED_BYTE, self.image)
						
						
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, self.wrap_s)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, self.wrap_t)
			
			borderColor = [1.0, 1.0, 1.0, 0.0]
			glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, borderColor)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, self.mag_filter)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, self.min_filter)
			self.setMustRebuild(False)
			#Once in OpenGL, we're good and don't have to reload it.
			self.image = None

	def deactivate(self):
		glBindTexture(GL_TEXTURE_2D, 0)
		glDisable(GL_TEXTURE_2D)
		
		
	def setMode(self, mode):
		self.mode = mode
		
			
	def getTexture_name(self):
		if (self.texture_id == None):
			list = []
			list = glGenTextures(1)
			#print list #Should generate a single ID - not a list.
			#if (len(list) != 1):			
			#	raise(Exception("No textures generated getTexture_name"))
			self.texture_id = list
			rebuild_texture = True
		return self.texture_id